@@ -8,7 +8,8 @@
 ##estimated.
 schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
                              fit.marge = FALSE, warn = TRUE, method = "BFGS",
-                             control = list(), std.err.type = "none", corr = FALSE){
+                             control = list(), std.err.type = "none", corr = FALSE,
+                             weights = NULL){
   ##data is a matrix with each column corresponds to one location
   ##locations is a matrix giving the coordinates (1 row = 1 station)
   n.site <- ncol(data)
@@ -17,9 +18,14 @@ schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
   n.pairs <- n.site * (n.site - 1) / 2
 
   dist <- distance(coord)
+  weighted <- !is.null(weights)
+
+  if (!weighted)
+    ##Set the weights to 0 as they won't be used anyway
+    weights <- 0
   
-  if (!(cov.mod %in% c("whitmat","cauchy","powexp","bessel")))
-    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp', 'bessel'")
+  if (!(cov.mod %in% c("whitmat","cauchy","powexp","bessel","caugen")))
+    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp', 'bessel', 'caugen'")
 
   if (cov.mod == "whitmat")
     cov.mod.num <- 1
@@ -29,34 +35,39 @@ schlatherindfull <- function(data, coord, start, cov.mod = "whitmat", ...,
     cov.mod.num <- 3
   if (cov.mod == "bessel")
     cov.mod.num <- 4
+  if (cov.mod == "caugen")
+    cov.mod.num <- 5
+
+  param <- c("alpha", "sill", "range", "smooth")
+
+  if (cov.mod == "caugen")
+    param <- c(param, "smooth2")
+  
+  else
+    ##Fix it to 0 as it won't be used anyway
+    smooth2 <- 0
+
+  if (fit.marge){
+    loc.names <- paste("loc", 1:n.site, sep="")
+    scale.names <- paste("scale", 1:n.site, sep="")
+    shape.names <- paste("shape", 1:n.site, sep="")
+    
+    param <- c(param, loc.names, scale.names, shape.names)
+  }
+
+  else
+    loc.names <- scale.names <- shape.names <- rep(1, n.site)
   
   ##First create a "void" function
   nplk <- function(x) x
 
   ##And define the "body" of the function as the number of parameters
   ##to estimate depends on n.site
-  if (fit.marge){
-    loc.names <- paste("loc", 1:n.site, sep="")
-    scale.names <- paste("scale", 1:n.site, sep="")
-    shape.names <- paste("shape", 1:n.site, sep="")
-    
-    param <- c("alpha", "sill", "range", "smooth", loc.names, scale.names, shape.names)
-
-    body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim),",
-                            paste("as.double(c(", paste(loc.names, collapse = ","), ")), "),
-                            paste("as.double(c(", paste(scale.names, collapse = ","), ")), "),
-                            paste("as.double(c(", paste(shape.names, collapse = ","), ")), "),
-                            "as.double(alpha), as.double(sill), as.double(range), as.double(smooth), fit.marge, dns = double(1), PACKAGE = 'SpatialExtremes')$dns"))
-  }
-
-  else{
-    body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim),",
-                            paste("as.double(rep(1,", n.site, ")), "),
-                            paste("as.double(rep(1,", n.site, ")), "),
-                            paste("as.double(rep(1,", n.site, ")), "),
-                            "as.double(alpha), as.double(sill), as.double(range), as.double(smooth), fit.marge, dns = double(1), PACKAGE = 'SpatialExtremes')$dns"))
-    param <- c("alpha", "sill", "range", "smooth")
-  }
+  body(nplk) <- parse(text = paste("-.C('schlatherindfull', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim), as.integer(weighted), as.double(weights),",
+                        paste("as.double(c(", paste(loc.names, collapse = ","), ")), "),
+                        paste("as.double(c(", paste(scale.names, collapse = ","), ")), "),
+                        paste("as.double(c(", paste(shape.names, collapse = ","), ")), "),
+                        "as.double(alpha), as.double(sill), as.double(range), as.double(smooth), as.double(smooth2), fit.marge, dns = double(1), PACKAGE = 'SpatialExtremes')$dns"))
 
   fixed.param <- list(...)[names(list(...)) %in% param]
 
@@ -76,12 +87,6 @@ Standard errors are not available unless you fix it.")
     std.err.type <- "none"
   }
   
-  if (std.err.type == "none")
-    hessian <- FALSE
-  
-  else
-    hessian <- TRUE
-  
   ##Define the formal arguments of the function
   form.nplk <- NULL
   for (i in 1:length(param))
@@ -171,7 +176,7 @@ Standard errors are not available unless you fix it.")
   
   if (method == "nlm"){
     start <- as.numeric(start)
-    opt <- nlm(nllh, start, hessian = hessian, ...)
+    opt <- nlm(nllh, start, ...)
     opt$counts <- opt$iterations
     names(opt$counts) <- "function"
     opt$value <- opt$minimum
@@ -192,8 +197,7 @@ Standard errors are not available unless you fix it.")
   }
 
   if (!(method %in% c("nlm", "nlminb"))){
-    opt <- optim(start, nllh, hessian = hessian, ..., method = method,
-                 control = control)
+    opt <- optim(start, nllh, ..., method = method, control = control)
   
     if ((opt$convergence != 0) || (opt$value >= 1.0e15)) {
       
@@ -217,35 +221,30 @@ Standard errors are not available unless you fix it.")
   param.names <- param
   param <- c(opt$par, unlist(fixed.param))
   param <- param[param.names]
+
+  ##Reset the weights to their original values
+  if ((length(weights) == 1) && (weights == 0))
+    weights <- NULL
   
   if (std.err.type != "none"){
+    std.err <- .schlatherindstderr(param, data, dist, cov.mod.num, as.double(0),
+                                   as.double(0), as.double(0), fit.marge = fit.marge,
+                                   std.err.type = std.err.type, fixed.param = names(fixed.param),
+                                   param.names = param.names, weights = weights)
+
+    opt$hessian <- std.err$hess
+    var.score <- std.err$var.score
+    ihessian <- try(solve(opt$hessian), silent = TRUE)
     
-    var.cov <- try(solve(opt$hessian), silent = TRUE)
-    if(!is.matrix(var.cov)){
+    if(!is.matrix(ihessian)){
       if (warn)
         warning("observed information matrix is singular; passing std.err.type to ''none''")
       
       std.err.type <- "none"
-      return
     }
 
-    else{
-      ihessian <- var.cov
-      jacobian <- .schlatherindgrad(param, data, dist, cov.mod.num, as.double(0),
-                                    as.double(0), as.double(0), fit.marge = fit.marge,
-                                    std.err.type = std.err.type, fixed.param = names(fixed.param),
-                                    param.names = param.names)
-
-      if(any(is.na(jacobian))){
-        if (warn)
-          warning("observed information matrix is singular; passing std.err.type to ''none''")
-        
-        std.err.type <- "none"
-      }
-    }
-
-    if (std.err.type != "none"){      
-      var.cov <- var.cov %*% jacobian %*% var.cov
+    else{  
+      var.cov <- ihessian %*% var.score %*% ihessian
       std.err <- diag(var.cov)
 
       std.idx <- which(std.err <= 0)
@@ -274,15 +273,21 @@ Standard errors are not available unless you fix it.")
 
   if (std.err.type == "none"){
     std.err <- std.err.type <- corr.mat <- NULL
-    var.cov <- ihessian <- jacobian <- NULL
+    var.cov <- ihessian <- var.score <- NULL
   }
 
-  cov.fun <-  covariance(sill = param["sill"], range = param["range"],
-                         smooth = param["smooth"], cov.mod = cov.mod, plot = FALSE)
+  if (cov.mod == "caugen")
+    cov.fun <-  covariance(sill = param["sill"], range = param["range"],
+                           smooth = param["smooth"], smooth2 = param["smooth2"],
+                           cov.mod = cov.mod, plot = FALSE)
+
+  else
+    cov.fun <-  covariance(sill = param["sill"], range = param["range"],
+                           smooth = param["smooth"], cov.mod = cov.mod, plot = FALSE)
   
   ext.coeff <- function(h)
     2 * param["alpha"] + (1 - param["alpha"]) *
-      (1 + sqrt(1 - 1/2 * (cov.fun(h) + 1)))
+      (1 + sqrt(0.5 - 0.5 * cov.fun(h)))
 
   fitted <- list(fitted.values = opt$par, std.err = std.err, std.err.type = std.err.type,
                  var.cov = var.cov, param = param, cov.fun = cov.fun, fixed = unlist(fixed.param),
@@ -291,7 +296,7 @@ Standard errors are not available unless you fix it.")
                  logLik = -opt$value, opt.value = opt$value, model = "Schlather",
                  cov.mod = cov.mod, fit.marge = fit.marge, ext.coeff = ext.coeff,
                  hessian = opt$hessian, lik.fun = nllh, coord = coord, ihessian = ihessian,
-                 jacobian = jacobian, marg.cov = NULL, nllh = nllh)
+                 var.score = var.score, marg.cov = NULL, nllh = nllh)
   
   class(fitted) <- c(fitted$model, "maxstab")
   return(fitted)
@@ -304,7 +309,7 @@ Standard errors are not available unless you fix it.")
 schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.form,
                              start, fit.marge = TRUE, marg.cov = NULL, ...,
                              warn = TRUE, method = "BFGS", control = list(),
-                             std.err.type = "none", corr = FALSE){
+                             std.err.type = "none", corr = FALSE, weights = NULL){
   ##data is a matrix with each column corresponds to one location
   ##coord is a matrix giving the coordinates (1 row = 1 station)
   n.site <- ncol(data)
@@ -313,9 +318,14 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
   n.pair <- n.site * (n.site - 1) / 2
 
   dist <- distance(coord)
-     
-  if (!(cov.mod %in% c("whitmat","cauchy","powexp","bessel")))
-    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp', 'bessel'")
+  weighted <- !is.null(weights)
+
+  if (!weighted)
+    ##Set the weights to 0 as they won't be used anyway
+    weights <- 0
+  
+  if (!(cov.mod %in% c("whitmat","cauchy","powexp","bessel","caugen")))
+    stop("''cov.mod'' must be one of 'whitmat', 'cauchy', 'powexp', 'bessel', 'caugen'")
 
   if (cov.mod == "whitmat")
     cov.mod.num <- 1
@@ -325,6 +335,8 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
     cov.mod.num <- 3
   if (cov.mod == "bessel")
     cov.mod.num <- 4
+  if (cov.mod == "caugen")
+    cov.mod.num <- 5
 
   ##With our notation, formula must be of the form y ~ xxxx
   loc.form <- update(loc.form, y ~ .)
@@ -373,18 +385,28 @@ schlatherindform <- function(data, coord, cov.mod, loc.form, scale.form, shape.f
   scale.names <- paste("scaleCoeff", 1:n.scalecoeff, sep="")
   shape.names <- paste("shapeCoeff", 1:n.shapecoeff, sep="")
   
-  param <- c("alpha", "sill", "range", "smooth", loc.names, scale.names, shape.names)
+  param <- c("alpha", "sill", "range", "smooth")
+
+  if (cov.mod == "caugen")
+    param <- c(param, "smooth2")
+
+  else
+    ##Fix it to 0 as it won't be used anyway
+    smooth2 <- 0
+  
+  param <- c(param, loc.names, scale.names, shape.names)
 
   ##First create a "void" function
   nplk <- function(x) x
 
   ##And define the "body" of the function as the number of parameters
   ##to estimate depends on n.site
-   body(nplk) <- parse(text = paste("-.C('schlatherinddsgnmat', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim), as.double(loc.dsgn.mat), as.double(loc.pen.mat), as.integer(n.loccoeff), as.integer(n.pparloc), as.double(loc.penalty), as.double(scale.dsgn.mat), as.double(scale.pen.mat), as.integer(n.scalecoeff), as.integer(n.pparscale), as.double(scale.penalty), as.double(shape.dsgn.mat), as.double(shape.pen.mat), as.integer(n.shapecoeff), as.integer(n.pparshape), as.double(shape.penalty),",
-                          paste("as.double(c(", paste(loc.names, collapse = ","), ")), "),
-                          paste("as.double(c(", paste(scale.names, collapse = ","), ")), "),
-                          paste("as.double(c(", paste(shape.names, collapse = ","), ")), "),
-                         "as.double(alpha), as.double(sill), as.double(range), as.double(smooth), dns = double(1), PACKAGE = 'SpatialExtremes')$dns"))
+  body(nplk) <- parse(text = paste("-.C('schlatherinddsgnmat', as.integer(cov.mod.num), as.double(data), as.double(dist), as.integer(n.site), as.integer(n.obs), as.integer(dist.dim), as.integer(weighted), as.double(weights), as.double(loc.dsgn.mat), as.double(loc.pen.mat), as.integer(n.loccoeff), as.integer(n.pparloc), as.double(loc.penalty), as.double(scale.dsgn.mat), as.double(scale.pen.mat), as.integer(n.scalecoeff), as.integer(n.pparscale), as.double(scale.penalty), as.double(shape.dsgn.mat), as.double(shape.pen.mat), as.integer(n.shapecoeff), as.integer(n.pparshape), as.double(shape.penalty),",
+                        paste("as.double(c(", paste(loc.names, collapse = ","), ")), "),
+                        paste("as.double(c(", paste(scale.names, collapse = ","), ")), "),
+                        paste("as.double(c(", paste(shape.names, collapse = ","), ")), "),
+                        "as.double(alpha), as.double(sill), as.double(range), as.double(smooth), as.double(smooth2), dns = double(1), PACKAGE = 'SpatialExtremes')$dns"))
+
   ##Define the formal arguments of the function
   form.nplk <- NULL
   for (i in 1:length(param))
@@ -442,12 +464,6 @@ Standard errors are not available unless you fix it.")
     std.err.type <- "none"
   }
   
-  if (std.err.type == "none")
-    hessian <- FALSE
-  
-  else
-    hessian <- TRUE
-
   if(any(!(param %in% c(nm,names(fixed.param)))))
     stop("unspecified parameters")
   
@@ -475,7 +491,7 @@ Standard errors are not available unless you fix it.")
   
   if (method == "nlm"){
     start <- as.numeric(start)
-    opt <- nlm(nllh, start, hessian = hessian, ...)
+    opt <- nlm(nllh, start, ...)
     opt$counts <- opt$iterations
     names(opt$counts) <- "function"
     opt$value <- opt$minimum
@@ -497,8 +513,7 @@ Standard errors are not available unless you fix it.")
   }
 
   if (!(method %in% c("nlm", "nlminb"))){
-    opt <- optim(start, nllh, hessian = hessian, ..., method = method,
-                 control = control)
+    opt <- optim(start, nllh, ..., method = method, control = control)
     
     if ((opt$convergence != 0) || (opt$value >= 1.0e15)){
       if (warn)
@@ -521,36 +536,31 @@ Standard errors are not available unless you fix it.")
   param.names <- param
   param <- c(opt$par, unlist(fixed.param))
   param <- param[param.names]
+
+  ##Reset the weights to their original values
+  if ((length(weights) == 1) && (weights == 0))
+    weights <- NULL
   
   if (std.err.type != "none"){
+    std.err <- .schlatherindstderr(param, data, dist, cov.mod.num, loc.dsgn.mat,
+                                   scale.dsgn.mat, shape.dsgn.mat,
+                                   fit.marge = fit.marge, std.err.type = std.err.type,
+                                   fixed.param = names(fixed.param), param.names =
+                                   param.names, weights = weights)
+
+    opt$hessian <- std.err$hess
+    var.score <- std.err$var.score
+    ihessian <- try(solve(opt$hessian), silent = TRUE)
     
-    var.cov <- try(solve(opt$hessian), silent = TRUE)
     if(!is.matrix(var.cov)){
       if (warn)
         warning("observed information matrix is singular; passing std.err.type to ''none''")
       
       std.err.type <- "none"
-      return
     }
 
-    else{
-      ihessian <- var.cov
-      jacobian <- .schlatherindgrad(param, data, dist, cov.mod.num, loc.dsgn.mat,
-                                    scale.dsgn.mat, shape.dsgn.mat,
-                                    fit.marge = fit.marge, std.err.type = std.err.type,
-                                    fixed.param = names(fixed.param), param.names =
-                                    param.names)
-
-      if(any(is.na(jacobian))){
-        if (warn)
-          warning("observed information matrix is singular; passing std.err.type to ''none''")
-        
-        std.err.type <- "none"
-      }
-    }
-
-    if (std.err.type != "none"){      
-      var.cov <- var.cov %*% jacobian %*% var.cov
+    else{    
+      var.cov <- ihessian %*% var.score %*% ihessian
       
       std.err <- diag(var.cov)
       
@@ -581,15 +591,21 @@ Standard errors are not available unless you fix it.")
 
   if (std.err.type == "none"){
     std.err <- std.err.type <- corr.mat <- NULL
-    var.cov <- ihessian <- jacobian <- NULL
+    var.cov <- ihessian <- var.score <- NULL
   }
 
-  cov.fun <- covariance(sill = param["sill"], range = param["range"],
-                        smooth = param["smooth"], cov.mod = cov.mod, plot = FALSE)
+  if (cov.mod == "caugen")
+    cov.fun <- covariance(sill = param["sill"], range = param["range"],
+                          smooth = param["smooth"], smooth2 = param["smooth2"],
+                          cov.mod = cov.mod, plot = FALSE)
+
+  else
+    cov.fun <- covariance(sill = param["sill"], range = param["range"],
+                          smooth = param["smooth"], cov.mod = cov.mod, plot = FALSE)
   
   ext.coeff <- function(h)
     2 * param["alpha"] + (1 - param["alpha"]) *
-      (1 + sqrt(1 - 1/2 * (cov.fun(h) + 1)))
+      (1 + sqrt(0.5 - 0.5 * cov.fun(h)))
   
   fitted <- list(fitted.values = opt$par, std.err = std.err, std.err.type = std.err.type,
                  var.cov = var.cov, fixed = unlist(fixed.param), param = param,
@@ -599,7 +615,7 @@ Standard errors are not available unless you fix it.")
                  fit.marge = fit.marge, ext.coeff = ext.coeff, cov.mod = cov.mod, cov.fun = cov.fun,
                  loc.form = loc.form, scale.form = scale.form, shape.form = shape.form,
                  lik.fun = nllh, loc.type = loc.type, scale.type = scale.type,
-                 shape.type = shape.type, ihessian = ihessian, jacobian = jacobian,
+                 shape.type = shape.type, ihessian = ihessian, var.score = var.score,
                  marg.cov = marg.cov, nllh = nllh)
   
   class(fitted) <- c(fitted$model, "maxstab")
