@@ -10,45 +10,46 @@
 //  From schlather.c
 //
 void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		   int *dim, double *locs, double *scales, double *shapes, double *sill,
-		   double *range, double *smooth, int *fitmarge, double *dns);
-void schlatherdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		      int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
-		      double *locpenalty, double *scaledsgnmat, double *scalepenmat,
-		      int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
-		      double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
-		      double *loccoeff, double *scalecoeff, double *shapecoeff, double *sill,
-		      double *range, double *smooth, double *dns);
+		   int *dim, int *weighted, double *weights, double *locs,
+		   double *scales, double *shapes, double *sill, double *range,
+		   double *smooth, double *smooth2, int *fitmarge, double *dns);
+void schlatherfull(int *covmod, double *data, double *dist, int *nSite, int *nObs,
+		   int *dim, int *weighted, double *weights, double *locs,
+		   double *scales, double *shapes, double *sill, double *range,
+		   double *smooth, double *smooth2, int *fitmarge,double *dns);
 
 ///////////////////////////////////
 //  From schlatherind.c
 //
 void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
-		      int *nObs, int *dim, double *locs, double *scales, double *shapes,
+		      int *nObs, int *dim, int *weighted, double *weights,
+		      double *locs, double *scales, double *shapes, 
 		      double *alpha, double *sill, double *range, double *smooth,
-		      int *fitmarge,double *dns);
+		      double *smooth2, int *fitmarge,double *dns);
 void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-			 int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
+			 int *dim, int *weighted, double *weights, double *locdsgnmat,
+			 double *locpenmat, int *nloccoeff, int *npparloc,
 			 double *locpenalty, double *scaledsgnmat, double *scalepenmat,
 			 int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
 			 double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
 			 double *loccoeff, double *scalecoeff, double *shapecoeff, double *alpha,
-			 double *sill, double *range, double *smooth, double *dns);
+			 double *sill, double *range, double *smooth, double *smooth2, double *dns);
 
 ///////////////////////////////////
 //  From geomgauss.c
 //
-void geomgaussfull(int *covmod, double *data, double *dist, int *nSite,
-		   int *nObs, int *dim, double *locs, double *scales, double *shapes,
-		   double *sigma2, double *sill, double *range, double *smooth,
-		   int *fitmarge,double *dns);
+void geomgaussfull(int *covmod, double *data, double *dist, int *nSite, int *nObs, int *dim,
+		   int *weighted, double *weights, double *locs, double *scales, double *shapes,
+		   double *sigma2, double *sigma2Bound, double *sill, double *range,
+		   double *smooth, double *smooth2, int *fitmarge,double *dns);
 void geomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-		      int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
-		      double *locpenalty, double *scaledsgnmat, double *scalepenmat,
-		      int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
-		      double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
-		      double *loccoeff, double *scalecoeff, double *shapecoeff, double *sigma2,
-		      double *sill, double *range, double *smooth, double *dns);
+		      int *dim, int *weighted, double *weights, double *locdsgnmat,
+		      double *locpenmat, int *nloccoeff, int *npparloc, double *locpenalty,
+		      double *scaledsgnmat, double *scalepenmat, int *nscalecoeff, int *npparscale,
+		      double *scalepenalty, double *shapedsgnmat, double *shapepenmat, int *nshapecoeff,
+		      int *npparshape, double *shapepenalty, double *loccoeff, double *scalecoeff,
+		      double *shapecoeff, double *sigma2, double *sigma2Bound, double *sill,
+		      double *range, double *smooth, double *smooth2, double *dns);
 
 ///////////////////////////////////
 //  From nsgeomgauss.c
@@ -56,7 +57,7 @@ void geomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *
 void nsgeomgaussfull(int *covmod, double *data, double *dist, int *nSite,
 		     int *nObs, int *dim, double *locs, double *scales, double *shapes,
 		     double *sigma2dsgnmat, double *sigma2coeff, int *nsigma2coeff,
-		     double *sill, double *range, double *smooth, int *fitmarge,
+		     double *sill, double *range, double *smooth, double *smooth2, int *fitmarge,
 		     double *dns);
 void nsgeomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
 			int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
@@ -65,33 +66,45 @@ void nsgeomgaussdsgnmat(int *covmod, double *data, double *dist, int *nSite, int
 			double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
 			double *sigma2dsgnmat, int *nsigma2coeff, double *loccoeff, double *scalecoeff,
 			double *shapecoeff, double *sigma2coeff, double *sill, double *range,
-			double *smooth, double *dns);
+			double *smooth, double *smooth2, double *dns);
+
+///////////////////////////////////
+//  From brownResnick.c
+//
+void brownresnickfull(double *data, double *dist, int *nSite, int *nObs, int *weighted,
+		      double *weights, double *locs, double *scales, double *shapes,
+		      double *range, double *smooth, int *fitmarge, double *dns);
+void brownresnickdsgnmat(double *data, double *dist, int *nSite, int *nObs, int *weighted,
+			 double *weights, double *locdsgnmat, double *locpenmat, int *nloccoeff,
+			 int *npparloc, double *locpenalty, double *scaledsgnmat, double *scalepenmat,
+			 int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
+			 double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
+			 double *loccoeff, double *scalecoeff, double *shapecoeff, double *range,
+			 double *smooth, double *dns);
 
 ///////////////////////////////////
 //  From smith.c
 //
-void smithfull(double *data, double *distVec, int *nSite,
-	       int *nObs, double *locs, double *scales, double *shapes,
-	       double *cov11, double *cov12, double *cov22, int *fitmarge,
-	       double *dns);
-void smithdsgnmat(double *data, double *distVec, int *nSite, int *nObs, 
-		  double *locdsgnmat, double *locpenmat, int *nloccoeff,
-		  int *npparloc, double *locpenalty, double *scaledsgnmat,
-		  double *scalepenmat, int *nscalecoeff, int *npparscale,
-		  double *scalepenalty, double *shapedsgnmat, double *shapepenmat,
-		  int *nshapecoeff, int *npparshape, double *shapepenalty,
-		  double *loccoeff, double *scalecoeff, double *shapecoeff,
-		  double *cov11, double *cov12, double *cov22, double *dns);
+void smithfull(double *data, double *distVec, int *nSite, int *nObs, int *weighted, double *weights,
+	       double *locs, double *scales, double *shapes, double *cov11, double *cov12,
+	       double *cov22, int *fitmarge, double *dns);
+void smithdsgnmat(double *data, double *distVec, int *nSite, int *nObs, int *weighted,
+		  double *weights, double *locdsgnmat, double *locpenmat, int *nloccoeff,
+		  int *npparloc, double *locpenalty, double *scaledsgnmat, double *scalepenmat,
+		  int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
+		  double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
+		  double *loccoeff, double *scalecoeff, double *shapecoeff, double *cov11,
+		  double *cov12, double *cov22, double *dns);
 
 ///////////////////////////////////
 //  From smith3d.c
 //
-void smithfull3d(double *data, double *distVec, int *nSite,
-		 int *nObs, double *locs, double *scales, double *shapes,
+void smithfull3d(double *data, double *distVec, int *nSite, int *nObs, int *weighted,
+		 double *weights, double *locs, double *scales, double *shapes,
 		 double *cov11, double *cov12, double *cov13, double *cov22,
 		 double *cov23, double *cov33, int *fitmarge, double *dns);
-void smithdsgnmat3d(double *data, double *distVec, int *nSite, int *nObs, 
-		    double *locdsgnmat, double *locpenmat, int *nloccoeff,
+void smithdsgnmat3d(double *data, double *distVec, int *nSite, int *nObs, int *weighted,
+		    double *weights, double *locdsgnmat, double *locpenmat, int *nloccoeff,
 		    int *npparloc, double *locpenalty, double *scaledsgnmat,
 		    double *scalepenmat, int *nscalecoeff, int *npparscale,
 		    double *scalepenalty, double *shapedsgnmat, double *shapepenmat,
@@ -135,6 +148,8 @@ double whittleMatern(double *dist, int nPairs, double sill, double range,
 		     double smooth, double *rho);
 double cauchy(double *dist, int nPairs, double sill, double range,
 	      double smooth, double *rho);
+double caugen(double *dist, int nPairs, double sill, double range,
+	      double smooth, double smooth2, double *rho);
 double powerExp(double *dist, int nPairs, double sill, double range,
 		double smooth, double *rho);
 double bessel(double *dist, int nPairs, int dim, double sill,
@@ -145,11 +160,14 @@ double mahalDistFct3d(double *distVec, int nPairs, double *cov11,
 		      double *cov12, double *cov13, double *cov22, 
 		      double *cov23, double *cov33, double *mahal);
 double geomCovariance(double *dist, int nPairs, int dim, int covmod,
-		      double sigma2, double sill, double range,
-		      double smooth, double *rho);
+		      double sigma2, double sigma2Bound, double sill,
+		      double range, double smooth, double smooth2,
+		      double *rho);
 double nsgeomCovariance(double *dist, int nSite, int dim, int covmod,
 			double *sigma2, double sill, double range,
-			double smooth, double *rho);
+			double smooth, double smooth2, double *rho);
+double brownResnick(double *dist, int nPairs, double range, double smooth,
+		    double *rho);
 
 ///////////////////////////////////
 //  From mcmc.c
@@ -157,44 +175,6 @@ double nsgeomCovariance(double *dist, int nSite, int dim, int covmod,
 SEXP gibbs(SEXP n, SEXP np, SEXP thin, SEXP init,
 	   SEXP psd, SEXP f, SEXP rho);
 
-///////////////////////////////////
-//  From gradients.c
-//
-void smithgrad(double *data, double *distVec, int *nSite,
-	       int *nObs, double *locdsgnmat, int *nloccoeff,
-	       double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
-	       int *nshapecoeff, double *loccoeff, double *scalecoeff,
-	       double *shapecoeff, double *cov11, double *cov12,
-	       double *cov22, int *fitmarge, double *grad);
-void smithgrad3d(double *data, double *distVec, int *nSite,
-		 int *nObs, double *locdsgnmat, int *nloccoeff,
-		 double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
-		 int *nshapecoeff, double *loccoeff, double *scalecoeff,
-		 double *shapecoeff, double *cov11, double *cov12, double *cov13,
-		 double *cov22, double *cov23, double *cov33, int *fitmarge, double *grad);
-void schlathergrad(int *covmod, double *data, double *dist, int *nSite,
-		   int *nObs, double *locdsgnmat, int *nloccoeff,
-		   double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
-		   int *nshapecoeff, double *loccoeff, double *scalecoeff,
-		   double *shapecoeff, double *sill, double *range, double *smooth,
-		   int *fitmarge, double *grad);
-void schlatherindgrad(int *covmod, double *data, double *dist, int *nSite,
-		      int *nObs, double *locdsgnmat, int *nloccoeff,
-		      double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
-		      int *nshapecoeff, double *loccoeff, double *scalecoeff,
-		      double *shapecoeff, double *alpha, double *sill, double *range,
-		      double *smooth, int *fitmarge, double *grad);
-void spatgevgrad(double *data, int *nSite, int *nObs, double *locdsgnmat,
-		 int *nloccoeff, double *scaledsgnmat, int *nscalecoeff,
-		 double *shapedsgnmat, int *nshapecoeff, double *loccoeff,
-		 double *scalecoeff, double *shapecoeff, double *grad);
-void geomgaussgrad(int *covmod, double *data, double *dist, int *nSite,
-		   int *nObs, double *locdsgnmat, int *nloccoeff,
-		   double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
-		   int *nshapecoeff, double *loccoeff, double *scalecoeff,
-		   double *shapecoeff, double *sigma2, double *sill, double *range,
-		   double *smooth, int *fitmarge, double *grad);
-
 ///////////////////////////////////
 //  From pairwiselik.c
 //
@@ -205,6 +185,16 @@ double lplikschlather(double *data, double *rho, double *jac,
 double lplikschlatherind(double *data, double alpha, double *rho,
 			 double *jac, int nObs, int nSite);
 
+///////////////////////////////////
+//  From weightedPairwiselik.c
+//
+double wlplikschlather(double *data, double *rho, double *jac,
+		       int nObs, int nSite, double *weights);
+double wlpliksmith(double *data, double *mahalDist, double *jac,
+		   int nObs, int nSite, double *weights);
+double wlplikschlatherind(double *data, double alpha, double *rho,
+			  double *jac, int nObs, int nSite, double *weights);
+
 ///////////////////////////////////
 //  From penalizations.c
 //
@@ -233,14 +223,18 @@ void fitcovmat3d(double *cov11, double *cov12, double *cov13,
 		 int *nPairs, double *dist, double *extcoeff,
 		 double *weights, double *ans);
 void fitcovariance(int *covmod, double *sill, double *range, double *smooth,
-		   int *nPairs, int *dim, double *distVec, double *extcoeff,
-		   double *weights, double *ans);
+		   double *smooth2, int *nPairs, int *dim, double *distVec,
+		   double *extcoeff, double *weights, double *ans);
 void fiticovariance(int *covmod, double *alpha, double *sill, double *range,
-		    double *smooth, int *nPairs, int *dim, double *dist, double *extcoeff,
-		    double *weights, double *ans);
-void fitgcovariance(int *covmod, double *sigma2, double *sill, double *range,
-		    double *smooth, int *nPairs, int *dim, double *dist, double *extcoeff,
-		    double *weights, double *ans);
+		    double *smooth, double *smooth2, int *nPairs, int *dim,
+		    double *dist, double *extcoeff, double *weights, double *ans);
+void fitgcovariance(int *covmod, double *sigma2, double *sigma2Bound, double *sill,
+		    double *range, double *smooth, double *smooth2, int *nPairs,
+		    int *dim, double *dist, double *extcoeff, double *weights,
+		    double *ans);
+void fitbrcovariance(double *range, double *smooth, int *nPairs,
+		     double *dist, double *extcoeff, double *weights,
+		     double *ans);
 
 ///////////////////////////////////
 //  From spatgevlik.c
@@ -320,3 +314,109 @@ void rgeomdirect(double *coord, int *nObs, int *nSite, int *dim,
 		 int *covmod, int *grid, double *sigma2, double *sill,
 		 double *range, double *smooth, double *uBound,
 		 double *ans);
+
+///////////////////////////////////
+//  From gpdproc.c
+//
+void gpdprocfull(double *data, double *distVec, int *nSite,
+		 int *nObs, double *excRates, double *threshs, double *scales,
+		 double *shapes, double *cov11, double *cov12,
+		 double *cov22, int *fitmarge, double *dns);
+double gpd2ugpd(double *data, int nObs, int nSite, double *excRates,
+		double *threshs, double *scales, double *shapes,
+		double *jac, double *ugpd);
+double lpliksmithgpd(double *data, double *mahalDist, double *jac,
+		     double *excRates, int nObs, int nSite);
+
+///////////////////////////////////
+//  From standardErrors.c
+//
+void smithstderr(double *data, double *distVec, int *nSite,
+		 int *nObs, double *locdsgnmat, int *nloccoeff,
+		 double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		 int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		 double *shapecoeff, double *cov11, double *cov12,
+		 double *cov22, int *fitmarge, double *hess, double *grad);
+void smithstderr3d(double *data, double *distVec, int *nSite,
+		   int *nObs, double *locdsgnmat, int *nloccoeff,
+		   double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		   int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		   double *shapecoeff, double *cov11, double *cov12, double *cov13,
+		   double *cov22, double *cov23, double *cov33, int *fitmarge, double *hess,
+		   double *grad);
+void schlatherstderr(int *covmod, double *data, double *dist, int *nSite,
+		     int *nObs, double *locdsgnmat, int *nloccoeff,
+		     double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		     int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		     double *shapecoeff, double *sill, double *range, double *smooth,
+		     double *smooth2, int *fitmarge, double *hess, double *grad);
+void schlatherindstderr(int *covmod, double *data, double *dist, int *nSite,
+			int *nObs, double *locdsgnmat, int *nloccoeff,
+			double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+			int *nshapecoeff, double *loccoeff, double *scalecoeff,
+			double *shapecoeff, double *alpha, double *sill, double *range,
+			double *smooth, double *smooth2, int *fitmarge, double *hess,
+			double *grad);
+void geomgaussstderr(int *covmod, double *data, double *dist, int *nSite,
+		     int *nObs, double *locdsgnmat, int *nloccoeff,
+		     double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		     int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		     double *shapecoeff, double *sigma2, double *sill, double *range,
+		     double *smooth, double *smooth2, int *fitmarge, double *hess,
+		     double *grad);
+void brownresnickstderr(double *data, double *dist, int *nSite, int *nObs,
+			double *locdsgnmat, int *nloccoeff, double *scaledsgnmat,
+			int *nscalecoeff, double *shapedsgnmat, int *nshapecoeff,
+			double *loccoeff, double *scalecoeff, double *shapecoeff,
+			double *range, double *smooth, int *fitmarge, double *hess,
+			double *grad);
+void spatgevstderr(double *data, int *nSite, int *nObs, double *locdsgnmat,
+		   int *nloccoeff, double *scaledsgnmat, int *nscalecoeff,
+		   double *shapedsgnmat, int *nshapecoeff, double *loccoeff,
+		   double *scalecoeff, double *shapecoeff, double *hess, 
+		   double *grad);
+
+///////////////////////////////////
+//  From weightedStandardErrors.c
+//
+void wsmithstderr(double *data, double *distVec, int *nSite,
+		  int *nObs, double *locdsgnmat, int *nloccoeff,
+		  double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		  int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		  double *shapecoeff, double *cov11, double *cov12,
+		  double *cov22, int *fitmarge, double *weights, double *hess,
+		  double *grad);
+void wsmithstderr3d(double *data, double *distVec, int *nSite,
+		    int *nObs, double *locdsgnmat, int *nloccoeff,
+		    double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		    int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		    double *shapecoeff, double *cov11, double *cov12, double *cov13,
+		    double *cov22, double *cov23, double *cov33, int *fitmarge,
+		    double *weights, double *hess, double *grad);
+void wschlatherstderr(int *covmod, double *data, double *dist, int *nSite,
+		      int *nObs, double *locdsgnmat, int *nloccoeff,
+		      double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		      int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		      double *shapecoeff, double *sill, double *range, double *smooth,
+		      double *smooth2, int *fitmarge, double *weights, double *hess,
+		      double *grad);
+void wschlatherindstderr(int *covmod, double *data, double *dist, int *nSite,
+			 int *nObs, double *locdsgnmat, int *nloccoeff,
+			 double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+			 int *nshapecoeff, double *loccoeff, double *scalecoeff,
+			 double *shapecoeff, double *alpha, double *sill, double *range,
+			 double *smooth, double *smooth2, int *fitmarge, double *weights,
+			 double *hess, double *grad);
+void wgeomgaussstderr(int *covmod, double *data, double *dist, int *nSite,
+		      int *nObs, double *locdsgnmat, int *nloccoeff,
+		      double *scaledsgnmat, int *nscalecoeff, double *shapedsgnmat,
+		      int *nshapecoeff, double *loccoeff, double *scalecoeff,
+		      double *shapecoeff, double *sigma2, double *sill, double *range,
+		      double *smooth, double *smooth2, int *fitmarge, double *weights,
+		      double *hess, double *grad);
+void wbrownresnickstderr(double *data, double *dist, int *nSite, int *nObs,
+			 double *locdsgnmat, int *nloccoeff, double *scaledsgnmat,
+			 int *nscalecoeff, double *shapedsgnmat, int *nshapecoeff,
+			 double *loccoeff, double *scalecoeff, double *shapecoeff,
+			 double *range, double *smooth, int *fitmarge, double *weights,
+			 double *hess, double *grad);
