@@ -1,9 +1,10 @@
 #include "header.h"
 
 void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
-		      int *nObs, int *dim, double *locs, double *scales, double *shapes,
+		      int *nObs, int *dim, int *weighted, double *weights,
+		      double *locs, double *scales, double *shapes, 
 		      double *alpha, double *sill, double *range, double *smooth,
-		      int *fitmarge,double *dns){
+		      double *smooth2, int *fitmarge,double *dns){
   //This is the independent Schlater's model. It's a wrapper to several
   //sub-functions. It's named xxxfull as it either assume that the
   //margins are unit Frechet, or the GEV parameters are estimated at
@@ -47,6 +48,9 @@ void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
   case 4:
     *dns = bessel(dist, nPairs, *dim, *sill, *range, *smooth, rho);
     break;
+  case 5:
+    *dns = caugen(dist, nPairs, *sill, *range, *smooth, *smooth2, rho);
+    break;
   }
   
   if (*dns != 0.0)
@@ -60,14 +64,22 @@ void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
     if (*dns != 0.0)
       return;
     
-    *dns = lplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite);
+    if (*weighted)
+      *dns = wlplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite, weights);
+
+    else
+      *dns = lplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite);
   }
   
   else {
     for (i=0;i<(*nSite * *nObs);i++)
       jac[i] = 0.0;
 
-    *dns = lplikschlatherind(data, *alpha, rho, jac, *nObs, *nSite);
+    if (*weighted)
+      *dns = wlplikschlatherind(data, *alpha, rho, jac, *nObs, *nSite, weights);
+
+    else
+      *dns = lplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite);
   }
   
   if (!R_FINITE(*dns))
@@ -78,12 +90,13 @@ void schlatherindfull(int *covmod, double *data, double *dist, int *nSite,
 }
 
 void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, int *nObs,
-			 int *dim, double *locdsgnmat, double *locpenmat, int *nloccoeff, int *npparloc,
+			 int *dim, int *weighted, double *weights, double *locdsgnmat,
+			 double *locpenmat, int *nloccoeff, int *npparloc,
 			 double *locpenalty, double *scaledsgnmat, double *scalepenmat,
 			 int *nscalecoeff, int *npparscale, double *scalepenalty, double *shapedsgnmat,
 			 double *shapepenmat, int *nshapecoeff, int *npparshape, double *shapepenalty,
 			 double *loccoeff, double *scalecoeff, double *shapecoeff, double *alpha,
-			 double *sill, double *range, double *smooth, double *dns){
+			 double *sill, double *range, double *smooth, double *smooth2, double *dns){
   //This is the independent Schlater's model.
   //The GEV parameters are defined using a polynomial response surface
   //or p-splines.
@@ -117,6 +130,9 @@ void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, in
   case 4:
     *dns = bessel(dist, nPairs, *dim, *sill, *range, *smooth, rho);
     break;
+  case 5:
+    *dns = caugen(dist, nPairs, *sill, *range, *smooth, *smooth2, rho);
+    break;
   }
 
   if (*dns != 0.0)
@@ -138,7 +154,11 @@ void schlatherinddsgnmat(int *covmod, double *data, double *dist, int *nSite, in
   if (*dns != 0.0)
     return;
 
-  *dns = lplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite);
+  if (*weighted)
+    *dns = wlplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite, weights);
+
+  else
+    *dns = wlplikschlatherind(frech, *alpha, rho, jac, *nObs, *nSite, weights);
     
   //Stage 5: Removing the penalizing terms (if any)
   // 1- For the location parameter
